import numpy as np
import pylab as plt
from merger_density_models import *
#plt.rc('font',size='16')


# - [ ] Computing 90% confidence interval for given weights
def computeLimits(x, W):
    #''' Given array x, and weights W, give median of X and min and max (1-sigma)
    
    #'''
    # Normalize W
    W = W / np.sum(W)# Sort by x
    indices = np.argsort(x)
    x = x[indices]
    W = W[indices]
    cumsums = np.cumsum(W)
    # Find the 1-sigma lower limit
    index = (np.abs(cumsums-0.05)).argmin()
    xlow  = x[index]
    # Find the 1-sigma upper limit
    index = (np.abs(cumsums-0.95)).argmin()
    xhigh = x[index]
    # Find the median
    index  = (np.abs(cumsums-0.5)).argmin()
    xmedian= x[index]
    # Return values
    ##print("here",xmedian,xlow,xhigh)
    return xmedian, xmedian-xlow, xhigh-xmedian


str1=np.array(["gal","clus"])
str2=np.array(["min","max"])

# Compute the 90% confidence region for the $\mathcal{R}(z)$

nlim=100
ncases=4
RzAfterLensing = np.zeros((ncases,3, nlim))
RzBeforeLensing = np.zeros((ncases,3, nlim))

for ii in range(str1.size):
    for jj in range (str2.size):
        Navgs1, kappa1, gamma1, zp1, R0par1, W1 = np.loadtxt("all_mdist_%s_rate_weights_mu%s.dat"%(str1[ii],str2[jj]),unpack=True)
        # Before lensing:
        zValues         = np.linspace(0,8,nlim)
        hh=jj+ii*str1.size
        for kk in range(nlim):
            zi         = zValues[kk]
            RzValueszi = Rz(zi, kappa1, gamma1, zp1, R0par1)
            # Get the 68% confidence interval
            noWeight = np.ones(len(RzValueszi))
            noWeight[W1==0] = 0
            RzMedian, RzLow, RzHigh = computeLimits(RzValueszi, noWeight)
            RzMin = RzMedian - RzLow
            RzMax = RzMedian + RzHigh
            # Save values
            RzBeforeLensing[hh,0,kk] = RzMedian
       #    print("Rzb:",hh,ii,jj,zi,RzValueszi.size,RzMedian,noWeight.size)
            RzBeforeLensing[hh,1,kk] = RzMin
            RzBeforeLensing[hh,2,kk] = RzMax
        # After lensing
            # Get the 68% confidence interval
            RzMedian, RzLow, RzHigh = computeLimits(RzValueszi, W1)
            RzMin = RzMedian - RzLow
            RzMax = RzMedian + RzHigh
            # Save values
            RzAfterLensing[hh,0,kk] = RzMedian
            RzAfterLensing[hh,1,kk] = RzMin
            RzAfterLensing[hh,2,kk] = RzMax


# Plot the results
#from latex import *

#
clabels=np.array(["GD","GS","CD","CS"])
fcol=np.array(["red","orange"])
lsty=np.array(["solid","dashed","dashdot","dotted"])

for ii in range(ncases):

    x      = zValues
    y1, y2 = RzBeforeLensing[ii,1,:], RzBeforeLensing[ii,2,:]
    if(ii==2):
        plt.fill_between(x, y1, y2, where=y2 >= y1, facecolor='lightblue', interpolate=True,alpha=0.5)
    plt.plot(x,y1, ls=lsty[ii], color='black')
    plt.plot(x,y2, ls=lsty[ii], color='black')

    x      = zValues
    y1, y2 = RzAfterLensing[ii,1,:], RzAfterLensing[ii,2,:]
    if(ii==2):
        plt.fill_between(x, y1, y2, where=y2 >= y1, facecolor=fcol[ii%2], interpolate=True,alpha=0.6)
    plt.plot(x,y1, ls=lsty[ii], color='black', label=clabels[ii])
    plt.plot(x,y2, ls=lsty[ii], color='black')

    if(ii==0):
       plt.text(2,2e6,"No lensing") #fontsize=16)
       plt.text(2,2e4,"Lensing") #,fontsize=16)
    

plt.legend(loc='lower right') #fontsize=16)
plt.xlim([0,8])
plt.ylim([10,5e6])
plt.xlabel(r"$z$")
plt.ylabel(r"$\mathcal{R}_{\rm m}($z$)$ [$\rm Gpc^{-3}$ $\rm yr^{-1}$]")
plt.yscale("log")
plt.grid()
plt.savefig(r"RateInference_MinMax.pdf", bbox_inches='tight')




